;
;  TCP/IP support routines
;****************************************************************************
;*                                                                          *
;*                                                                          *
;*      part of NCSA Telnet                                                 *
;*      by Tim Krauskopf, VT100 by Gaige Paulsen, Tek by Aaron Contorer     *
;*                                                                          *
;*      National Center for Supercomputing Applications                     *
;*      152 Computing Applications Building                                 *
;*      605 E. Springfield Ave.                                             *
;*      Champaign, IL  61820                                                *
;*                                                                          *
;****************************************************************************
;
	NAME	IPASM
	INCLUDE	DOS.MAC
	SETX

	PSEG
	PUBLIC	IPCHECK,TCPCHECK,MOVEBYTE,LONGSWAP,INTSWAP,DBG
;
;  Routines for general use by the communications programs
;
;
;************************************************************************
;  DBG
;  provides a synch point for debugging
;
dbg	proc	far
	nop
nop
nop
ret
dbg endp
;
;*************************************************************************
;  Internet header checksum
;    header checksum is calculated for a higher level program to verify
;
;  USAGE:  ipcheck(ptr)
;
;  this proc knows that the IP header length is found in the first byte
;
IPCHECK	PROC	FAR
	PUSH	BP
	MOV	BP,SP
	PUSH	DS
	MOV	AX,[BP+X+2]	; ds for input data
	MOV	DS,AX
	MOV	SI,[BP+X]	; pointer to data
	MOV	CX,[BP+X+4]	; count of words to test

	XOR	BX,BX
	CLC
CHKSUM:
	LODSW			; get next word
	ADC	BX,AX		; keep adding
	LOOP	CHKSUM		; til' done

	ADC	BX,0		; adds the carry bit in
				;
	NOT	BX		; take one more 1-complement

	MOV	AX,BX
	POP	DS
	POP	BP
	RET
IPCHECK	ENDP
;
;  TCP checksum, has two parts, including support for a pseudo-header
;
;  usage:   tcpcheck(psptr,tcpptr,tcplen)
;            char *psptr,*tcpptr;  pointers to pseudo header and real header
;            int tcplen            length of tcp packet in checksum
;

TCPCHECK	PROC	FAR
	PUSH	BP
	MOV	BP,SP
	PUSH	DS
	MOV	AX,[BP+X+2]	; ds for input data for pseudo-hdr
	MOV	DS,AX
	MOV	SI,[BP+X]	; pointer to data
	MOV	CX,6		; length of p-hdr in words

	XOR	BX,BX           ; clear to begin
	CLC
PCHKSUM:
	LODSW			; get next word
	ADC	BX,AX		; keep adding
	LOOP	PCHKSUM		; til' done

	ADC	BX,0		; adds the carry bit in
;
; NOW THE REAL THING
;
	MOV	AX,[BP+X+6]	; ds of real stuff
	MOV	DS,AX
	MOV	SI,[BP+X+4]	; pointer

	MOV	CX,[BP+X+8]	; count of bytes to test
	MOV	DX,CX		; keep a copy
	SHR	CX,1		; divide by two, round down

	CLC
RCHKSUM:
	LODSW
	ADC	BX,AX		; add to previous running sum
	LOOP	RCHKSUM
	ADC	BX,0		; add the last carry in again

	AND	DX,1		; odd # of bytes?
	JZ	NOTODD
	LODSB			; get that last byte
	XOR	AH,AH		; clear the high portion
	ADD	BX,AX		; add the last one in
	ADC	BX,0		; add the carry in, too

NOTODD:
	NOT	BX		; take one more 1-complement

	MOV	AX,BX
	POP	DS
	POP	BP
	RET
TCPCHECK	ENDP

;
;********************************************************************
;  New movebytes
;  Move an arbitrary number of bytes from one location to another.
;  Thanks to Russell Nelson for his contribution to this code.
;
;  Usage:
;  movebytes(to,from,count)
;   char *to,*from;
;   int16 count
;   moves < 64K from one 4 byte pointer to another.  Does not handle
;   overlap, but does copy quickly.
;

MOVEBYTE	PROC	FAR
	PUSH	BP
	MOV	BP,SP
	PUSH	DS
	PUSH	ES
	MOV	AX,[BP+X+2]	; WHERE TO PUT IT
	MOV	ES,AX
	MOV	AX,[BP+X+6]	; WHERE TO GET IT
	MOV	DS,AX
	MOV	SI,[BP+X+4]	; FROM PTR
	MOV	DI,[BP+X]	; TO PTR
	MOV	CX,[BP+X+8]	; HOW MANY TO MOVE

	shr cx,1		; make into a word count
	jnc dowords
	movsb			; move the odd byte
dowords:
	rep movsw

	POP	ES
	POP	DS
	POP	BP
	RET

MOVEBYTE	ENDP

;
;*************************************************************************
;  longswap
;    swap the bytes of a long integer from PC
;  order (reverse) to in-order.  This will work both ways.
;  returns the new long value
;  usage:
;      l2 = longswap(l)
;	long l;
;
LONGSWAP	PROC	FAR
	PUSH	BP
	MOV	BP,SP
	MOV	BX,[BP+X+2]	; high bytes of the long int
	MOV	AX,[BP+X]	; low bytes of the long int
;
;  GET THE DATA
;
	XCHG	AH,AL		; SWAP THEM, these are now low
	XCHG	BH,BL		; swap the others
	POP	BP
	RET
LONGSWAP	ENDP

;
;*************************************************************************
;  INTSWAP
;    swap the bytes of an integer, returns the swapped integer
;
;   usage:      i = intswap(i);
;
INTSWAP	PROC	FAR
	MOV	BX,SP
	MOV	AX,SS:[BX+4]	; CAN IT BE DONE?
	XCHG	AH,AL
	RET
;	PUSH	BP
;	MOV	BP,SP
;	MOV	AX,[BP+X]	; GET THE INT
;	XCHG	AH,AL		; SWITCH THEM, THAT'S IT
;	POP	BP
;	RET
INTSWAP	ENDP

;
;**************************************************************************
;
;  Routines to install and deinstall a timer routine which calls
;  netsleep(0);
;  The timer is set to go off every 1/2 second to check for packets 
;  in the incoming packet buffer.  We use the user-hook into the system 
;  timer which occurs every 1/18th of a second.
;
;
TIMEINT	EQU	4*1CH		; User hook to timer int

	EXTRN	netsleep:FAR	; C routine which gets called from handler
	PUBLIC	TINST,TDEINST

;*************************************************************************
;
;  Take out the timer interrupt handler, restore previous value
;
TDEINST	PROC	FAR
	MOV	CX,CS:TIP	; get old ip from save spot
	MOV	DX,CS:TCS	; get old cs from save spot
	MOV	BX,TIMEINT	; interrupt in table for timer
	PUSH	DS
	XOR	AX,AX		; system interrupt table
	MOV	DS,AX		
	CLI
	MOV	[BX],CX		; store old ip into the table
	INC	BX
	INC	BX		; move pointer in interrupt table
	MOV	[BX],DX		; store old cs into the table
	STI
	POP	DS
	RET
TDEINST	ENDP
;
;
;  install the timer interrupt handler, the handler is technically
;  part of this procedure.
;
TINST	PROC	FAR
	XOR	AX,AX
	MOV	CS:TENTER,AL	; clear this flag
	MOV	CS:TMYDS,DS	; store for use by handler
	MOV	BX,TIMEINT	; interrupt in table for timer (1C)
	PUSH	DS
	XOR	AX,AX		; system interrupt table
	MOV	DS,AX		
	MOV	AX,OFFSET THAND	; where the handler is
	CLI
	MOV	DX,[BX]		; keep copy of the ip
	MOV	[BX],AX		; store ip into the table
	INC	BX
	INC	BX		; move pointer in interrupt table
	MOV	CX,[BX]		; keep copy of the cs, too
	MOV	AX,CS
	MOV	[BX],AX		; store new cs into the table
	STI
	POP	DS
	MOV	CS:TIP,DX	; store them away
	MOV	CS:TCS,CX
	RET
;
;  Code segment addressable data for keeping track of the interrupt handler
;  stuff
;
TMYDS	DW	00H		; the data segment for this assembly code
TICNT	DB	0		; counter of 1/18ths sec
TENTER DB  00
TIP  DW  00
TCS  DW  00
;
;   The handler itself.
;
THAND:			       	; not a public name, only handles ints
	STI
	PUSH	DS
	PUSH 	ES
	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	DI
	PUSH	SI
	CLD			; all moves will be forward
	MOV	AL,CS:TENTER
	OR	AL,AL
	JNZ	TIME2
	MOV	AL,1
	MOV	CS:TENTER,AL		; set flag to indicate busy
	INC	CS:TICNT
	MOV	AL,CS:TICNT		; counter for us
	AND	AL,7			; see if # mod 8 = 0
	JNZ	TSKIP			; skip 7 out of 8 times
;
	mov	al,60h			; EOI for timer int
	out	20h,al			; let lower interrupts in
;
;  SET UP CORRECT DS
;
	MOV	DS,CS:TMYDS	; get correct ds
;
;  do we have to set up our own stack here?
;
	xor	ax,ax
	PUSH AX
	CALL netsleep
	POP	AX

TSKIP:
	XOR	AL,AL
	MOV	CS:TENTER,AL		; reenter flag, done now
TIME2:
	POP SI
	POP	DI
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	POP	ES
	POP	DS
;
;   forward to any other existing routines
;
	JMP DWORD PTR CS:TIP
TINST	ENDP


	ENDPS
	END
