;
;  Changes to get this from NET.ASM
;    1. X EQU 6
;    2. replace "include DOS.MAC"
;    3. replace DSEG, PSEG and endings
;    4. put underscores in front of all public identifiers
;    5. fill out some identifiers to full length
;    6. Check returns of long pointers and longs to use DX-low, AX-hi
;
	NAME	NET
;	INCLUDE	DOS.MAC
;	SETX
X	EQU	6

;
;   The subroutines to call from C
;
_TEXT	segment	public	'CODE'
	assume CS:_TEXT
	PUBLIC	_IPCHECK,_TCPCHECK,_MOVEBYTES,_MOVENBYTES,_LONGSWAP,_INTSWAP

;
;  Routines for general use by the communications programs
;

;
;*************************************************************************
;  Internet header checksum
;    header checksum is calculated for a higher level program to verify
;
;  USAGE:  ipcheck(ptr)
;
;  this proc knows that the IP header length is found in the first byte
;
_IPCHECK	PROC	FAR
	PUSH	BP
	MOV	BP,SP
	PUSH	DS
	MOV	AX,[BP+X+2]	; ds for input data
	MOV	DS,AX
	MOV	SI,[BP+X]	; pointer to data
	MOV	CX,[BP+X+4]	; count of words to test

	XOR	BX,BX
	CLC
CHKSUM:
	LODSW			; get next word
	ADC	BX,AX		; keep adding
	LOOP	CHKSUM		; til' done

	ADC	BX,0		; adds the carry bit in
				;
	NOT	BX		; take one more 1-complement

	MOV	AX,BX
	POP	DS
	POP	BP
	RET
_IPCHECK	ENDP
;
;  TCP checksum, has two parts, including support for a pseudo-header
;
;  usage:   tcpcheck(psptr,tcpptr,tcplen)
;            char *psptr,*tcpptr;  pointers to pseudo header and real header
;            int tcplen            length of tcp packet in checksum
;

_TCPCHECK	PROC	FAR
	PUSH	BP
	MOV	BP,SP
	PUSH	DS
	MOV	AX,[BP+X+2]	; ds for input data for pseudo-hdr
	MOV	DS,AX
	MOV	SI,[BP+X]	; pointer to data
	MOV	CX,6		; length of p-hdr in words

	XOR	BX,BX           ; clear to begin
	CLC
PCHKSUM:
	LODSW			; get next word
	ADC	BX,AX		; keep adding
	LOOP	PCHKSUM		; til' done

	ADC	BX,0		; adds the carry bit in
;
; NOW THE REAL THING
;
	MOV	AX,[BP+X+6]	; ds of real stuff
	MOV	DS,AX
	MOV	SI,[BP+X+4]	; pointer

	MOV	CX,[BP+X+8]	; count of bytes to test
	MOV	DX,CX		; keep a copy
	SHR	CX,1		; divide by two, round down

	CLC
RCHKSUM:
	LODSW
	ADC	BX,AX		; add to previous running sum
	LOOP	RCHKSUM
	ADC	BX,0		; add the last carry in again

	AND	DX,1		; odd # of bytes?
	JZ	NOTODD
	LODSB			; get that last byte
	XOR	AH,AH		; clear the high portion
	ADD	BX,AX		; add the last one in
	ADC	BX,0		; add the carry in, too

NOTODD:
	CMP	BX,0ffffH	; don't not it to 0 (special case: all 1's)
	JZ	NONOT	
	NOT	BX		; take one more 1-complement
NONOT:

	MOV	AX,BX
	POP	DS
	POP	BP
	RET
_TCPCHECK	ENDP
;
;*********************************************************************8
;  movebytes
;   ONLY MOVES EVEN NUMBERS OF BYTES!!!! WILL DROP THE LAST BYTE OF
;   ODD BYTES!
;   use movenbytes for arbitrary byte moves
;   usage:
;     movebytes(to,from,n);       n must be even
;
_MOVEBYTES	PROC	FAR
	PUSH	BP
	MOV	BP,SP
	PUSH	DS
	PUSH	ES
	MOV	AX,[BP+X+2]	; WHERE TO PUT IT
	MOV	ES,AX
	MOV	AX,[BP+X+6]	; WHERE TO GET IT
	MOV	DS,AX
	MOV	SI,[BP+X+4]	; FROM PTR
	MOV	DI,[BP+X]	; TO PTR
	MOV	CX,[BP+X+8]	; HOW MANY TO MOVE
	SHR	CX,1		; DIVIDE BY TWO WORDS

	REP	MOVSW		; MOVE THEM

	POP	ES
	POP	DS
	POP	BP
	RET
_MOVEBYTES	ENDP

;*********************************************************************8
;  movenbytes
;   use movenbytes for arbitrary byte moves of any count
;   usage:
;     movenbytes(to,from,n);  
;
_MOVENBYTES	PROC	FAR
	PUSH	BP
	MOV	BP,SP
	PUSH	DS
	PUSH	ES
	MOV	AX,[BP+X+2]	; WHERE TO PUT IT
	MOV	ES,AX
	MOV	AX,[BP+X+6]	; WHERE TO GET IT
	MOV	DS,AX
	MOV	SI,[BP+X+4]	; FROM PTR
	MOV	DI,[BP+X]	; TO PTR
	MOV	CX,[BP+X+8]	; HOW MANY TO MOVE
	OR	CX,CX		; COULD BE NONE TO MOVE
	JZ	ENDMOVE
	CMP	CX,1		; SEE IF THERE IS JUST ONE BYTE
	JZ	LASTONE
	MOV	BX,CX		; MAKE A COPY TO KEEP
	AND	BX,1		; SEE IF IT IS ODD
	JNZ	ODDN

	SHR	CX,1		; DIVIDE BY TWO WORDS

	REP	MOVSW		; MOVE THEM

	POP	ES
	POP	DS
	POP	BP
	RET

ODDN:
	SHR	CX,1		; DIVIDE BY TWO
	
	REP	MOVSW
LASTONE:
	MOVSB			; MOVE ODD ONE
ENDMOVE:
	POP	ES
	POP	DS
	POP	BP
	RET
_MOVENBYTES	ENDP
;
;*************************************************************************
;  longswap
;    swap the bytes of a long integer from PC
;  order (reverse) to in-order.  This will work both ways.
;  returns the new long value
;  usage:
;      l2 = longswap(l)
;	long l;
;
_LONGSWAP	PROC	FAR
	PUSH	BP
	MOV	BP,SP
	MOV	AX,[BP+X+2]	; high bytes of the long int
	MOV	DX,[BP+X]	; low bytes of the long int
;
;  GET THE DATA
;
	XCHG	AH,AL		; SWAP THEM, these are now low
	XCHG	DH,DL		; swap the others
	POP	BP
	RET
_LONGSWAP	ENDP

;
;*************************************************************************
;  INTSWAP
;    swap the bytes of an integer, returns the swapped integer
;
;   usage:      i = intswap(i);
;
_INTSWAP	PROC	FAR
	MOV	BX,SP
	MOV	AX,SS:[BX+4]	; CAN IT BE DONE?
	XCHG	AH,AL
	RET
;	PUSH	BP
;	MOV	BP,SP
;	MOV	AX,[BP+X]	; GET THE INT
;	XCHG	AH,AL		; SWITCH THEM, THAT'S IT
;	POP	BP
;	RET
_INTSWAP	ENDP


;
;**************************************************************************
;
;  Routines to install and deinstall a timer routine which calls
;  netsleep(0);
;  The timer is set to go off every 1/2 second to check for packets 
;  in the incoming packet buffer.  We use the user-hook into the system 
;  timer which occurs every 1/18th of a second.
;
;
TIMEINT	EQU	4*1CH		; User hook to timer int

	EXTRN	_netsleep:FAR	; C routine which gets called from handler
	PUBLIC	_TINST,_TDEINST

;*************************************************************************
;
;  Take out the timer interrupt handler, restore previous value
;
_TDEINST	PROC	FAR
	MOV	CX,CS:TIP	; get old ip from save spot
	MOV	DX,CS:TCS	; get old cs from save spot
	MOV	BX,TIMEINT	; interrupt in table for timer
	PUSH	DS
	XOR	AX,AX		; system interrupt table
	MOV	DS,AX		
	CLI
	MOV	[BX],CX		; store old ip into the table
	INC	BX
	INC	BX		; move pointer in interrupt table
	MOV	[BX],DX		; store old cs into the table
	STI
	POP	DS
	RET
_TDEINST	ENDP
;
;
;  install the timer interrupt handler, the handler is technically
;  part of this procedure.
;
_TINST	PROC	FAR
	XOR	AX,AX
	MOV	CS:TENTER,AL	; clear this flag
	MOV	CS:TMYDS,DS	; store for use by handler
	MOV	BX,TIMEINT	; interrupt in table for timer (1C)
	PUSH	DS
	XOR	AX,AX		; system interrupt table
	MOV	DS,AX		
	MOV	AX,OFFSET THAND	; where the handler is
	CLI
	MOV	DX,[BX]		; keep copy of the ip
	MOV	[BX],AX		; store ip into the table
	INC	BX
	INC	BX		; move pointer in interrupt table
	MOV	CX,[BX]		; keep copy of the cs, too
	MOV	AX,CS
	MOV	[BX],AX		; store new cs into the table
	STI
	POP	DS
	MOV	CS:TIP,DX	; store them away
	MOV	CS:TCS,CX
	RET
;
;  Code segment addressable data for keeping track of the interrupt handler
;  stuff
;
TMYDS	DW	00H		; the data segment for this assembly code
TICNT	DB	0		; counter of 1/18ths sec
TENTER DB  00
TIP  DW  00
TCS  DW  00
;
;   The handler itself.
;
THAND:			       	; not a public name, only handles ints
	STI
	PUSH	DS
	PUSH 	ES
	PUSH	AX
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	DI
	PUSH	SI
	CLD			; all moves will be forward
	MOV	AL,CS:TENTER
	OR	AL,AL
	JNZ	TIME2
	MOV	AL,1
	MOV	CS:TENTER,AL		; set flag to indicate busy
	INC	CS:TICNT
	MOV	AL,CS:TICNT		; counter for us
	AND	AL,7			; see if # mod 8 = 0
	JNZ	TSKIP			; skip 7 out of 8 times
;
;  SET UP CORRECT DS
;
	MOV	DS,CS:TMYDS	; get correct ds
;
;  do we have to set up our own stack here?
;
	xor	ax,ax
	PUSH AX
	CALL _netsleep
	POP	AX

TSKIP:
	XOR	AL,AL
	MOV	CS:TENTER,AL		; reenter flag, done now
TIME2:
	POP SI
	POP	DI
	POP	DX
	POP	CX
	POP	BX
	POP	AX
	POP	ES
	POP	DS
;
;   forward to any other existing routines
;
	JMP DWORD PTR CS:TIP
_TINST	ENDP

_TEXT	ends

	END
