;
;  ncsaio.asm
;  Support for BIOS calls in NCSA Telnet
;****************************************************************************
;*                                                                          *
;*                                                                          *
;*      part of NCSA Telnet                                                 *
;*      by Tim Krauskopf, VT100 by Gaige Paulsen, Tek by Aaron Contorer     *
;*                                                                          *
;*      National Center for Supercomputing Applications                     *
;*      152 Computing Applications Building                                 *
;*      605 E. Springfield Ave.                                             *
;*      Champaign, IL  61820                                                *
;*                                                                          *
;*                                                                          *
;****************************************************************************

	TITLE	NCSAIO    -- LOW-LEVEL I/O FOR SANE HARDWARE HANDLING
;Microsoft EQU 1
Lattice EQU 1
  ifndef Microsoft
    ifndef Lattice
      if2
        %out
        %out ERROR: You have to specify "/DMicrosoft" OR "/DLattice" on the
        %out        MASM command line to determine the type of assembly.
        %out
      endif
      end
    endif
  endif

;
;   From original code by Tim Krauskopf    1984-1985
;
;   Modified and ported to Lattice C, Sept. 1986
;   ifdefs for Microsoft C, June 1987
;   Tim Krauskopf
;
;   National Center for Supercomputing Applications
;
	NAME	NIO

;
;  Macros for reading and writing I/O ports
;
MOUT	MACRO	REG,STUFF       ; one byte to the given I/O register
	MOV	DX,REG
	MOV	AL,STUFF
	OUT	DX,AL
	ENDM
;
MIN	MACRO	REG         	; get one byte to al
	MOV	DX,REG
	IN	AL,DX
	ENDM
;
;  Internal data 
;
	X EQU	6				;  for the large model programs

ifdef Microsoft
DGROUP	group	_DATA
_DATA	segment	public 'DATA'
	assume	DS:DGROUP

	public _dtaptr				; pointer to dta location
_dtaptr	dw	0000h				; DTA address for me
_dtads	dw	0000h				; DS for DTA
else
	INCLUDE	DOS.MAC
	DSEG

	public	dtaptr				; pointer to dta location

dtaptr	dw	0000h				; DTA address for me
dtads	dw	0000h				; DS for DTA
endif

att		db	7					; current default attribute
topl	db	00					; top line of current window
botl	db	24					; bottom line of current window
leftc	db	00					; left side of current window
rightc	db	79					; right side of current window
row		db	00					; cursor position row
col		db	00					; cursor position col
wrap	db	00					; wrap at end or not
regen	dw	0b000h				; default to color screen
;
;    In this implementation, all special and function keys are translated
;    to the following values.
;
tr	db	3 dup (0)
	db  0dbh
	db  11 dup(0)
	db	09bh,0d1h,0d7h,0c5h,0d2h,0d4h,0d9h,0d5h,0c9h,0cfh,0d0h
	db	4 dup(0)
	db	0c1h,0d3h,0c4h,0c6h,0c7h,0c8h,0cah,0cbh,0cch
	db	5 dup(0)
	db	0dah,0d8h,0c3h,0d6h,0c2h,0ceh,0cdh
	db	8 dup(0)
	db	80h,81h,82h,83h,84h,85h,86h,87h,88h,89h
	db	0,0
	db	8ah,8dh,9ah,0,8bh,0,9bh,0,8ch,8eh,9ch,9dh,9eh
	db	90h,91h,92h,93h,94h,95h,96h,97h,98h,99h
	db	0a0h,0a1h,0a2h,0a3h,0a4h,0a5h,0a6h,0a7h,0a8h,0a9h
	db	0e0h,0e1h,0e2h,0e3h,0e4h,0e5h,0e6h,0e7h,0e8h,0e9h
	db	0aeh,0abh,0bbh,0ach,0bch,0aah
	db	0b0h,0b1h,0b2h,0b3h,0b4h,0b5h,0b6h,0b7h,0b8h,0b9h
	db	0bdh,0beh,0bah
	db  124 dup(0)

ifdef Microsoft
_DATA	ends
else
	ENDDS
endif
;
;
;
;   The subroutines to call from C
;
ifdef Microsoft
_TEXT	segment	public	'CODE'
	PUBLIC	_n_color,_n_upscroll,_n_dnscroll,_n_wrap,_n_erase,_n_getchar
	PUBLIC  _n_cur,_n_row,_n_col,_n_clear,_n_window,_n_putchar,_n_chkchar
	PUBLIC  _n_savewin,_n_restwin,_n_puts,_n_sound,_n_findfirst,_n_findnext
	PUBLIC  _n_draw,_n_scrup,_n_scrdn,_n_cheat,_n_scrlck,_n_clicks
	assume CS:_TEXT
else
	PSEG

	PUBLIC	n_color,n_upscro,n_dnscro,n_wrap,n_erase,n_getcha
	PUBLIC  n_cur,n_row,n_col,n_clear,n_window,n_putcha,n_chkcha
	PUBLIC  n_savewi,n_restwi,n_puts,n_sound,n_findfi,n_findne
	PUBLIC  n_draw,n_scrup,n_scrdn,n_cheat,n_scrlck,n_clicks
endif

;/***************************************************************/
; scrlck
;   returns whether scroll lock is on or not
;
ifdef Microsoft
_n_scrlck	proc	far
else
n_scrlck	proc	far
endif
		push bp
		mov		ax,0200h		; get shift states
		int		16h				; keyboard int
		and		al,010h			; scroll lock state bit
		xor		ah,ah
		pop bp
		ret	
ifdef Microsoft
_n_scrlck	endp
else
n_scrlck	endp
endif


;/***************************************************************/
;
;   Change the value of my current color
;
ifdef Microsoft
_n_color	proc	far
else
n_color	proc	far
endif
		push	bp
		mov		bp,sp
		xor		ax,ax
		mov		dl,[bp+x]			; parameter, one byte
		mov		al,att				; return old color
		mov		att,dl
		pop		bp
		ret
ifdef Microsoft
_n_color	endp
else
n_color	endp
endif


;/*****************************************************************/
;   Determine the current type of display
;
;   ask the BIOS which screen is currently active
;
ifdef Microsoft
_n_which	proc	far
else
n_which	proc	far
endif
	push bp
	mov	ah,15			; get screen mode
	int	10h				; call video BIOS
	cmp	al,7			; is it mono?
	jz	K1				; yes
    mov ax,0b800h		; no, set for color
    jmp K2
K1: mov ax,0b000h
K2: 
    mov  regen,ax
	pop bp
    ret
ifdef Microsoft
_n_which	endp
else
n_which	endp
endif

;
;/***************************************************************/
;
;   Set whether word wrap will be on or not
;   usage:  n_wrap(flag);
;
ifdef Microsoft
_n_wrap	proc	far
else
n_wrap	proc	far
endif
		push	bp
		mov		bp,sp
		mov		dl,[bp+x]			; parameter, one byte flag
		mov		al,wrap				; return old color
		mov		wrap,dl
		pop		bp
		ret
ifdef Microsoft
_n_wrap	endp
else
n_wrap	endp
endif

;/***************************************************************/
;
;   Move the cursor somewhere
;
ifdef Microsoft
_n_cur	proc	far
else
n_cur	proc	far
endif
		push	bp
		mov		bp,sp
		mov		dh,[bp+x]			; row position
		mov		dl,[bp+x+2]			; column position
		mov		row,dh				; save a copy for me
		mov		col,dl				; save column too
	    mov  	ax,0200h			; ah=2 function call
	    xor 	bx,bx				; video page 0
	    int  	10h      			; set cursor position
		pop		bp
		ret
ifdef Microsoft
_n_cur	endp
else
n_cur	endp
endif

;/***************************************************************/
;
;  set the window boundaries
;
;  usage:  n_window(ulrow,ulcol,lrrow,lrcol);
;     sets window size for future operations.
;
;
ifdef Microsoft
_n_window	proc	far
else
n_window	proc	far
endif
		push	bp
		mov		bp,sp
		mov		al,[bp+x]			; upper left x
		mov		topl,al
		mov		al,[bp+x+2]			; column position
		mov		leftc,al			; save a copy for me
		mov		al,[bp+x+4]
		mov		botl,al				; bottom row
		mov		al,[bp+x+6]
		mov		rightc,al			; keep this one too

ifdef Microsoft
		call	_n_which				; what kind of screen?
else
		call	n_which				; what kind of screen?
endif
		pop		bp
		ret
ifdef Microsoft
_n_window	endp
else
n_window	endp
endif


;/***************************************************************/
;
;  erase portion of the screen
;
;  usage:  n_erase(ulrow,ulcol,lrrow,lrcol);
;
;
ifdef Microsoft
_n_erase	proc	far
else
n_erase	proc	far
endif
		push	bp
		mov		bp,sp
		mov		ch,[bp+x]			; upper left x
		mov		cl,[bp+x+2]			; column position
		mov		dh,[bp+x+4]
		mov		dl,[bp+x+6]
		xor		ax,ax				; clear area
		mov		ah,6				; scroll up function
		mov		bh,att				; attribute for erasing is same as print
		int		10h					; call BIOS

		pop		bp
		ret
ifdef Microsoft
_n_erase	endp
else
n_erase	endp
endif

;/***************************************************************/
;  scroll up and down
;  given the number of lines to scroll and the dimensions of the
;  box to scroll.
;  
ifdef Microsoft
_n_scrup	proc	far
else
n_scrup	proc	far
endif
		push	bp
		mov		bp,sp
		mov		ch,[bp+x+2]			; upper left x
		mov		cl,[bp+x+4]			; column position
		mov		dh,[bp+x+6]
		mov		dl,[bp+x+8]
		mov		al,[bp+x]			; number of lines to scroll
		mov		ah,6				; scroll up function
		mov		bh,att				; attribute for blank line is same as print
		int		10h					; call BIOS

		pop		bp
		ret
ifdef Microsoft
_n_scrup	endp
else
n_scrup	endp
endif

ifdef Microsoft
_n_scrdn	proc	far
else
n_scrdn	proc	far
endif
		push	bp
		mov		bp,sp
		mov		ch,[bp+x+2]			; upper left x
		mov		cl,[bp+x+4]			; column position
		mov		dh,[bp+x+6]
		mov		dl,[bp+x+8]
		mov		al,[bp+x]			; number of lines to scroll
		mov		ah,7				; scroll down function
		mov		bh,att				; attribute for blank line is same as print
		int		10h					; call BIOS

		pop		bp
		ret
ifdef Microsoft
_n_scrdn	endp
else
n_scrdn	endp
endif

;/***************************************************************/
; 
;  Find the cursor position, return the row value
;  
ifdef Microsoft
_n_row	proc	far
else
n_row	proc	far
endif
		push bp
	    mov ax,0300h    	        ;	video find cursor function
	    xor bx,bx
	    int 10h  		            ; find cursor
		mov		row,dh
		mov		col,dl				; save col for next routine
		xor		ax,ax
		mov		al,dh				; return row location
		pop bp
		ret
ifdef Microsoft
_n_row	endp
else
n_row	endp
endif

;/****************************************************************************/
  ;-----------------------------------------------------------
  ;  This routine was submitted by David T. Burhans, Jr.
  ;  It was obtained from an RBBS-PC electronic bulleton board 
  ;  dedicated to the "C" language [(703) 321-7003].
  ;-----------------------------------------------------------
  ;
  ; SOUND - This routine produces a tone of a specified frequency 
  ; and duration on the speaker.
  ; The frequency in Hertz is moved into DI (21 to 65535 Hertz) 
  ; and the duration in hundredths of a second, is moved into BX 
  ; (0 to 65535).
  ; The routine is called with the following sequence:
  ;
  ;        Unsigned        frequency, duration;
  ;
  ;       n_sound(frequency, duration);
  ;
ifdef Microsoft
_n_sound	proc	far
else
n_sound	proc	far
endif
       push     bp
       mov      bp,sp
       mov      di,[bp+X]        ; Frequency
       mov      bx,[bp+X+2]      ; Duration
       cmp      di, 21H          ; See if freq. is below minimum
       jb       done              ; If less than limit, then
                                  ; clean up and return
       mov      al,0B6H           ; Write timer mode register
       out      43H, al
       mov      dx, 14H           ; Timer divisor =
       mov      ax, 4F38H         ; 1331000/Frequency
       div      di
       out      42H,al            ; Write timer 2 count low byte
       mov      al,ah
       out      42H,al            ; Write timer 2 count high byte
       in       al,61H            ; Get current Port B setting
       mov      ah,al             ; and save it in AH
       or       al,3              ; Turn speaker on
       out      61H,al
hold:  mov      cx, 2801
spkr_on: loop   spkr_on
       dec      bx                ; Speaker-on count expired?
       jnz      hold              ; If not, keep speaker on
       mov      al,ah             ; Otherwise, recover value of port
       out      61H,al
done:  mov      sp,bp             ; Restore locals from stack.
       pop      bp                ; Restore caller's bp
       ret                        ; Return to caller

ifdef Microsoft
_n_sound	endp
else
n_sound	endp
endif

;/***************************************************************/
;
;  Find the column position of the cursor.
;  Must be preceded by a n_row call!!!!
;  Will work when these routines already know the cursor position
;
ifdef Microsoft
_n_col	proc	far
else
n_col	proc	far
endif
		xor		ah,ah
		mov		al,col				; return col location
		ret
ifdef Microsoft
_n_col	endp
else
n_col	endp
endif

;/***************************************************************/
;  n_clear
;  Clear what we think is the current window
;
ifdef Microsoft
_n_clear	proc	far
else
n_clear	proc	far
endif
		push bp
		mov		al,0				; clear window
		mov		ah,6				; scroll up function
		mov		ch,topl				; top row to clear
		mov		cl,leftc			; left column
		mov		dh,botl				; bottom row to clear
		mov		dl,rightc			; right side to clear
		mov		bh,att				; attribute to use for blanks
		int		10h					; call BIOS
		pop bp
		ret
ifdef Microsoft
_n_clear	endp
else
n_clear	endp
endif


;/***************************************************************/
;  n_upscroll
;  Scroll up what we think is the current window, n lines
;
ifdef Microsoft
_n_upscroll	proc	far
else
n_upscro	proc	far
endif
		push	bp
		mov		bp,sp
		mov		al,[bp+x]			; number of lines to scroll
		mov		ah,6				; scroll up function
		mov		ch,topl				; top row to clear
		mov		cl,leftc			; left column
		mov		dh,botl				; bottom row to clear
		mov		dl,rightc			; right side to clear
		mov		bh,att				; attribute to use
		int		10h					; call BIOS
		pop		bp
		ret
ifdef Microsoft
_n_upscroll	endp
else
n_upscro	endp
endif

;/***************************************************************/
;  n_dnscroll
;  Scroll up what we think is the current window, n lines
;
ifdef Microsoft
_n_dnscroll	proc	far
else
n_dnscro	proc	far
endif
		push	bp
		mov		bp,sp
		mov		al,[bp+x]			; number of lines to scroll
		mov		ah,7				; scroll down function
		mov		ch,topl				; top row to clear
		mov		cl,leftc			; left column
		mov		dh,botl				; bottom row to clear
		mov		dl,rightc			; right side to clear
		mov		bh,att				; attribute to use for blank lines
		int		10h					; call BIOS
		pop		bp
		ret
ifdef Microsoft
_n_dnscroll	endp
else
n_dnscro	endp
endif


;/***************************************************************/
;  n_putchar(letter)
;      puts onto screen at current cursor location
;
ifdef Microsoft
_n_putchar	proc	far
else
n_putcha	proc	far
endif
		push bp
        mov bp,sp
        mov al,[bp+x]   		; char to write
	    xor  bx,bx       		;  set page number for all cursor addresses

	    cmp  al,10           	; line feed
	    jnz nxt

		mov		al,row			; get current cursor row position
		cmp		al,botl
		jl		noscr			; within window area

		mov		ax,1			; scroll up one line
		push	ax
ifdef Microsoft
		call	_n_upscroll		; scrolls it up one
else
		call	n_upscro		; scrolls it up one
endif
		pop		ax				; take parameter off of stack
;		mov		al,leftc		; get left side value
;		mov		col,al			; set cursor to left
		jmp		putcur

noscr:
;		mov		al,leftc		; return cursor to left side
;		mov		col,al
		inc		row
		jmp		putcur			; set new cursor position, return

nxt:
		cmp  al,7 				; ctrl-G, bell
	    jnz  nxt2
;
;  handle bell with a call to a special routine
;
	mov ax,12        ; duration of tone
	push ax
	mov ax,1000      ; frequency of tone
	push ax
ifdef Microsoft
	call _n_sound
else
	call n_sound
endif
	add sp,4         ; remove params from stack

here2:
		pop		bp
		ret

nxt2:
	    cmp		al,13           ; cr,  home the cursor
		jnz		trytab
		mov		al,leftc		; new cursor position
		mov		col,al
		jmp		putcur

trytab:
	    cmp  	al,9            ; tab character
	    jnz  notcrlf   			; ready for regular character

;  expand tabs
		mov		dl,col			; get cursor position
		mov		cl,3
		shr 	dl,cl			; divide cursor position by 8
		inc		dl				; increment cursor position
		shl		dl,cl			; multiply back by 8
		mov		col,dl

							; check to see if past right side
		mov		dl,col		; get where the cursor has moved to
		cmp		dl,rightc	; over the side yet?
		ja		tabover		; set the new position
		jmp		putcur

tabover:
		mov		dl,leftc
		mov		col,dl

		inc		row			; to next row
		mov		dl,row		; what row?
		cmp		dl,botl
		jg		tabnos		; we are okay
		jmp		putcur

tabnos:
		dec		row			; need to scroll

		mov		ax,1
		push	ax
ifdef Microsoft
		call	_n_upscroll	; scroll window up one line
else
		call	n_upscro	; scroll window up one line
endif
		pop		ax
		mov		al,leftc	; reset cursor position to left
		mov		col,al
	    jmp  	putcur

notcrlf:
    	cmp  	al,8           ; backspace
	    jnz 	regchar

		mov		al,col			; where is cursor?
		cmp		al,leftc		; is at left of screen?
		jz		here2
		dec		col				; decrement cursor position
		jmp		putcur			; okay, move cursor where it belongs

regchar:
	    mov  cx,1h         		; number of repetitions = 1
   		mov  bl,att 			; attribute of char
	    mov  ah,9         		; write char
   		int  10h

	    inc  	col             	; move cursor over one. 

		mov		dl,col			; get current position
    	cmp  	dl,rightc		; is at right side of screen?
	    jle   	putcur	 		; no, char can just be put out, cursor moved.

;
;  check word-wrap because we are at edge of window
;
		mov		dl,wrap
		or		dl,dl			; 0 = no wrap, 1 = wrap
		jz		nowrap

    	mov  	dl,leftc		; cursor will wrap around
		mov		col,dl			; save the new column position
		inc		row				; to next row
		mov		dl,botl
	    cmp  	row,dl			; do we need to scroll?

	    jng  putcur				; no, okay for next row, normal wrap

; scroll screen up one 
		push	ax
		mov		ax,1
		push	ax
ifdef Microsoft
		call	_n_upscroll	; scroll window up one line
else
		call	n_upscro	; scroll window up one line
endif
		pop		ax
		pop		ax				; get the character back
		dec		row				; scrolled up one

		jmp		putcur			; don't wrap

nowrap:
		dec		col

putcur:
		mov		dh,row
		mov		dl,col
    	mov  	ah,2h
    	xor 	bx,bx
	    int  	10h             ; set cursor position

escapehere:
    pop bp
    ret

tele:
    mov bl,3              ; send whatever char
    mov ah,14
    int 10h
    ret

ifdef Microsoft
_n_putchar	endp
else
n_putcha	endp
endif

;/**********************************************************************/
;  draw
;  place characters on the screen-- checking for bounds, etc.  all done
;  at a higher level.
;
;  n_draw(s,len)
;    char*s ; int len;
;
ifdef Microsoft
_n_draw proc far
else
n_draw proc far
endif
	push bp
	mov bp,sp
	push es
;
;  for now, assume that the cursor location has already been set
;  (cursor is there)
;
	mov	si,[bp+x]				; pointer to string
	mov	ax,[bp+x+2]				; ds of string
	mov es,ax
	mov	di,[bp+x+4]				; # of characters
	mov	dh,row					; row where cursor is

loopdraw:
	mov	al,es:[si]				; get character to write from es:si
	inc	si
    mov  cx,1h           		; number of repetitions = 1
	xor  bh,bh					; display page 0
   	mov  bl,att 				; attribute of char
    mov  ah,9           		; write char
   	int  10h

	inc	col						; move the cursor over
	mov		dl,col
   	mov  	ah,2h
	xor		bx,bx
    int  	10h             	; set cursor position

	dec di						; check counter
	jnz loopdraw				; go through to required count

	pop es
	pop	bp
	ret
ifdef Microsoft
_n_draw endp
else
n_draw endp
endif

;/**********************************************************************/
;  cheat
;  put characters on the screen as per n_draw, but write directly
;  to display memory.  Don't even check for retrace
;
ifdef Microsoft
_n_cheat proc	far
else
n_cheat proc	far
endif
	push bp
	mov bp,sp
	push es
	push ds
;
;  for now, assume that the cursor location has already been set
;  (cursor is there)
;
	mov	ax,regen
	mov	es,ax					; set up for segment where screen is

	mov	al,row					; row where chars go
	xor	ah,ah
	mov	bl,160
	mul	bl						; ax now equals start of row
	xor	bh,bh
	mov	bl,col
	shl	bl,1					; col*2
	add	ax,bx					; add in column offset
	mov	di,ax					; store in di (es:di)

	mov	cx,[bp+x+4]				; # of characters
	add	col,cl					; column position of cursor
	mov	ah,att					; attribute for chars
	mov	si,[bp+x]				; pointer to string
	mov	bx,[bp+x+2]				; ds of string
	mov ds,bx
;
;  do the move
;
	cld
lpcheat:
	lodsb						; get next byte
	stosw						; store byte and attribute
	loop lpcheat

	pop	ds
	pop	es
	pop	bp
	ret

ifdef Microsoft
_n_cheat	endp
else
n_cheat	endp
endif

;/**********************************************************************/
;
;  keyboard handling

;    the interrupt and codes for the keyboard interface.

keyboard    equ    16h        ;interrupt 16 to deal with keyboard

getc      	equ    0        ;code for reading a character
checkc	    equ    1        ;code for keyboard status


;**********************************************************************
;
;  get a translated character from the keyboard.
;  Why hasn't someone else written this first?
;  Tim Krauskopf
;
ifdef Microsoft
_n_getchar	proc	far
else
n_getcha	proc	far
endif

	               		;return the next character pressed
    	                ; translate if necessary
        push  	bp
        mov    	ah,getc    ;ask for a keyboard character
        int    	keyboard
        or		al,al
        jnz    	notextend		; is it an extended char?

        mov    bx,offset dgroup:tr		; convert special key
		mov		al,ah
		xlatb					; translate through table

notextend:
	    mov    ah,0			; char returns in al
        pop    bp
        ret
ifdef Microsoft
_n_getchar	endp
else
n_getcha	endp
endif


;**********************************************************************
;  Check for character present at the keyboard
;  If there is one available, return it, if not, return -1
;
;  translate any extended characters
;
ifdef Microsoft
_n_chkchar	proc	far
else
n_chkcha	proc	far
endif
        push    bp
        mov    ah,checkc
        int    keyboard
        mov    ax,-1
        jz    	nokey
ifdef Microsoft
        call    _n_getchar        ;get the coded character
else
        call    n_getcha        ;get the coded character
endif
nokey:
    	pop    bp
        ret
ifdef Microsoft
_n_chkchar	endp
else
n_chkcha	endp
endif


;/***************************************************************/
;  savewin
;    copy the current window into a buffer
;
;   usage:  n_savewindow(buffer);
;
ifdef Microsoft
_n_savewin	proc	far
else
n_savewi	proc	far
endif
		push	bp
		mov		bp,sp
		push	es
		push	ds
		mov		ax,[bp+x+2]			; ds of buffer
		mov		es,ax
		mov		di,[bp+x]			; pointer to buffer
		cld
;
;  store parameters in first, for recall later
;
		mov		al,row				; cursor position: row,col
		stosb
		mov		al,col
		stosb
		mov		al,topl				; window boundaries
		stosb
		mov		al,leftc
		stosb
		mov		al,botl
		stosb
		mov		al,rightc
		stosb						; store in buffer too
;
;  calculate amount to move
;
;
;  calculate number of lines to move
;
        mov  al,topl              ; x1
		mov  bl,botl				; x2
		sub  bl,al                  ; x2 = x2-x1
        inc  bl                     ; add 1 line
		mov  [bp+x],bl				; keep it somewhere safe
;
;  calculate screen offset  = 160*x1+2*y1;
;
;   al has topl in it
		xor  ah,ah
        mov  bl,160
        mul  bl					; how many chars in x1 lines, in ax
		xor  ch,ch
        mov  cl,leftc           ; chars to left side
        shl  cl,1               ; *2 counts attributes
        add  ax,cx              ; add them
        mov  si,ax           	; place into source index
;
;  find number of characters to move each time
;
		mov  cl,rightc
        sub  cl,leftc        ; y2 = y2-y1
        inc  cl              ;  add 1
		mov  [bp+x+1],cl		; safe place again
;
;  find number to add to wrap around for each line
;
        mov  bl,cl           ; # of chars moved each time
        shl  bl,1            ; chars*2
        mov  al,160
        sub  al,bl           ; 160-chars*2
		mov  [bp+x+2],al     ; safe place to keep it
;
;
        mov  ax,regen        ; where source segment is (screen)
        mov  ds,ax

        mov  dx,03dah             ; color card status port
		xor  bh,bh
        mov  bl,[bp+x]           ; howmany lines to move

        cmp  ax,0b000h            ; check for mono card
        jnz getline
        mov  dx,03bah             ; change status port value
getline:
        mov  cl,[bp+x+1]           ; # of chars to move
getchar:
;        in   al,dx                ; get status byte
;        test al,1                 ; horizontal retrace
;        jnz  getchar              ; not there yet
;ison:   in   al,dx
;        test al,1
;        jz   ison                 ; still on, try again
        movsw                     ; now is time, move the word
        loop getchar              ; do another until this line is done
;
        dec bx
        jz  endkeep               ; done, we are out of here
		mov	al,[bp+x+2]
		xor	ah,ah
        add si,ax                 ; go to next line, skip 160-n*2 bytes
        jmp getline

endkeep:
        pop  ds
        pop  es
        pop  bp
        ret

ifdef Microsoft
_n_savewin	endp
else
n_savewi	endp
endif


;/***************************************************************/
;  restwindow
;    restore the contents of the window to the screen
;
ifdef Microsoft
_n_restwin	proc	far
else
n_restwi	proc	far
endif
		push	bp
		mov		bp,sp
		push	es
		push	ds
		mov		ax,[bp+x+2]			; ds of buffer
		push	ax					; will pop to ds later 
		mov		es,ax				; used to restore variable values
		mov		si,[bp+x]			; pointer to buffer
		cld
;
;  get back stored variables
;
		mov		al,es:[si]			; get cursor row
		mov		row,al
		inc		si
		mov		al,es:[si]
		mov		col,al
		inc		si
		mov		al,es:[si]			; window boundaries
		mov		topl,al
		inc		si
		mov		al,es:[si]
		mov		leftc,al
		inc		si
		mov		al,es:[si]
		mov		botl,al
		inc		si
		mov		al,es:[si]
		mov		rightc,al
		inc		si					; now we are ready for data

;
;  calculate amount to move
;
;
;  calculate number of lines to move
;
        mov  al,topl              ; x1
		mov  bl,botl				; x2
		sub  bl,al                  ; x2 = x2-x1
        inc  bl                     ; add 1 line
		mov  [bp+x],bl				; keep it somewhere safe
;
;  calculate screen offset  = 160*x1+2*y1;
;
;   al has topl in it
		xor  ah,ah
        mov  bl,160
        mul  bl					; how many chars in x1 lines, in ax
		xor  ch,ch
        mov  cl,leftc           ; chars to left side
        shl  cl,1               ; *2 counts attributes
        add  ax,cx              ; add them
        mov  di,ax           	; place into source index
;
;  find number of characters to move each time
;
		mov  cl,rightc
        sub  cl,leftc        ; y2 = y2-y1
        inc  cl              ;  add 1
		mov  [bp+x+1],cl		; safe place again
;
;  find number to add to wrap around for each line
;
        mov  bl,cl           ; # of chars moved each time
        shl  bl,1            ; chars*2
        mov  al,160
        sub  al,bl           ; 160-chars*2
		mov  [bp+x+2],al     ; safe place to keep it
;
;
        mov  ax,regen        ; screen will receive this data
        mov  es,ax
		pop  ds				; set ds to where data is coming from 

        mov  dx,03dah             ; color card status port
		xor  bh,bh
        mov  bl,[bp+x]           ; howmany lines to move

        cmp  ax,0b000h            ; check for mono card
        jnz rgetline
        mov  dx,03bah             ; change status port value
rgetline:
        mov  cl,[bp+x+1]           ; # of chars to move
rgetchar:
;        in   al,dx                ; get status byte
;        test al,1                 ; horizontal retrace
;        jnz  rgetchar              ; not there yet
;rison:   in   al,dx
;        test al,1
;        jz   rison                 ; still on, try again
        movsw                     ; now is time, move the word
        loop rgetchar              ; do another until this line is done
;
        dec bx
        jz  rendkeep               ; done, we are out of here
		mov	al,[bp+x+2]
		xor	ah,ah
        add di,ax                 ; go to next line, skip 160-n*2 bytes
        jmp rgetline

rendkeep:
        pop  ds

		mov		dh,row				; load stored cursor position
		mov		dl,col
    	mov  	ah,2h
    	xor 	bx,bx
	    int  	10h             ; set cursor position to saved value

        pop  es
        pop  bp
        ret

ifdef Microsoft
_n_restwin	endp
else
n_restwi	endp
endif

;************************************************************************
;  n_puts
;  Window-compatible puts, uses n_putchar and also translates
;  \n to CRLF
;
;     usage:  identical to puts()
;
ifdef Microsoft
_n_puts	proc	far
else
n_puts	proc	far
endif
		push	bp
		mov		bp,sp
nextc:
		push	ds
		mov		dx,[bp+x+2]			; ds of string
		mov		ds,dx
		mov		bx,[bp+x]			; ptr to string
		mov		al,[bx]				; get character
		xor		ah,ah				; clear ah
		pop		ds
		or		al,al				; is it end of string?
		jz		donest

		inc		word ptr [bp+x]		; increment pointer for next one

		cmp		al,10				; newline?
		jnz		dochar
		mov		al,13
		push	ax
ifdef Microsoft
		call	_n_putchar
else
		call	n_putcha
endif
		pop		ax
		mov		ax,10
dochar:
		push	ax
ifdef Microsoft
		call	_n_putchar
else
		call	n_putcha
endif
		pop		ax					; take off of the stack
		jmp		nextc

donest:
		mov		ax,13				; CR
		push	ax
ifdef Microsoft
		call	_n_putchar
else
		call	n_putcha
endif
		pop		ax
		mov		ax,10				; LF
		push	ax
ifdef Microsoft
		call	_n_putchar
else
		call	n_putcha
endif
		pop		ax

		pop		bp
		ret
ifdef Microsoft
_n_puts	endp
else
n_puts	endp
endif

;**********************************************************************
;
;  find first
;    make dos find file names according to wildcards
;  n_findfirst(filename,attr)
;	char *filename; int attr;
;
ifdef Microsoft
_n_findfirst	proc	far
else
n_findfi	proc	far
endif
	push	bp
	mov		bp,sp
	push	es
	push	ds
	mov		ah,02fh					; DOS function get DTA
	int		21h
ifdef Microsoft
	mov		_dtaptr,bx				; squirrel a copy for me
	mov		ax,es
	mov		_dtads,ax
else
	mov		dtaptr,bx				; squirrel a copy for me
	mov		ax,es
	mov		dtads,ax
endif

	mov		ax,[bp+x+2]				; ds of filename ptr
	mov		ds,ax
	mov		dx,[bp+x]				; ptr part of filename
	mov		cx,[bp+x+4]				; attribute to search for
	mov		ah,04eh					; find matching file DOS call
	int		21h
	jc		badret					; ax already contains error code
	xor		ax,ax

badret:
	pop		ds
	pop		es
	pop		bp
	ret
ifdef Microsoft
_n_findfirst	endp
else
n_findfi	endp
endif
;
;  n_findnext()
;  will find entries that follow findfirst
;  no need to respecify file name
;
ifdef Microsoft
_n_findnext	proc	far
else
n_findne	proc	far
endif
	push	bp
	mov		ah,04fh					; find next DOS call
	int		21h
	jc		nbadret
	xor		ax,ax

nbadret:
	pop		bp
	ret
ifdef Microsoft
_n_findnext	endp
else
n_findne	endp
endif

;
;  get the number of timer clicks from BIOS
;
ifdef Microsoft
_n_clicks  proc	far            ; must be declared long n_clicks()
else
n_clicks  proc	far            ; must be declared long n_clicks()
endif
	mov		ah,0
	int		1ah
ifdef Microsoft
	mov		ax,dx				; MSC uses AX-lo, DX-hi
	mov		dx,cx              ; return values from interrupt 1A
else
	mov		ax,cx				; Lattice uses AX-hi, BX-lo
	mov		bx,dx				; Lattice returns in BX, MSC in DX and switched
endif
	ret
ifdef Microsoft
else
endif
ifdef Microsoft
_n_clicks endp
else
n_clicks endp
endif

ifdef Microsoft
_TEXT	ends
else
	endps
endif

	end

