/*
*  pctools.c
****************************************************************************
*                                                                          *
*      part of:                                                            *
*      TCP/UDP/ICMP/IP Network kernel for NCSA Telnet                      *
*      by Tim Krauskopf                                                    *
*                                                                          *
*      National Center for Supercomputing Applications                     *
*      152 Computing Applications Building                                 *
*      605 E. Springfield Ave.                                             *
*      Champaign, IL  61820                                                *
*                                                                          *
****************************************************************************
*
*  those generic tool-type things that only work on PCs.
*  includes all hardware-level calls to Ethernet that are unique to the PC
*
*  Function pointers for Ether calls
*/
#include "stdio.h"
#include "protocol.h"
#include "data.h"

/*
*   defined in assembly language file for interrupt driven Ether buffering
*
*/
unsigned char stat;                    /* status from last read */
char *bufpt,*bufend,*bufread,*buforg;
int bufbig,buflim;

/*
*  Declare each and every Ethernet driver.
*  To add a driver, pick a unique 2 char prefix and declare your own
*  routines.  I want to keep the same parameters for EVERY driver.
*  If your driver needs additional parameters, then see netconfig() below 
*  for an indication of where to put custom board code.
*/
extern int E1etopen(),E1getaddr(),E1setaddr(),E1recv(),E1xmit(),E1etupdate(),E1etclose();
extern int E3etopen(),E3getaddr(),E3setaddr(),E3recv(),E3xmit(),E3etupdate(),E3etclose();
extern int M5etopen(),M5getaddr(),M5recv(),M5xmit(),M5etupdate(),M5etclose();
extern int U1etopen(),U1getaddr(),U1recv(),U1xmit(),U1etupdate(),U1etclose();
extern int U2etopen(),U2getaddr(),U2recv(),U2xmit(),U2etupdate(),U2etclose();
extern int WDetopen(),WDgetaddr(),WDrecv(),WDxmit(),WDetupdate(),WDetclose();
extern int E2etopen(),E2getaddr(),E2recv(),E2xmit(),E2etupdate(),E2etclose();

static int (*etopen)()=NULL,	/* open the device */
	(*getaddr)()=NULL,		/* get the Ether address */
	(*setaddr)()=NULL,			/* set the Ether address to use */
	(*recv)()=NULL,				/* load a packet from queue */
	(*etupdate)()=NULL,			/* update pointers in buffer */
	(*etclose)()=NULL,			/* shut down network */
	(*xmit)()=NULL;				/* transmit a packet */


/**********************************************************************/
/*  statcheck
*   look at the connection status of the memory buffers to see if the
*   allocation schemes are working.  Only used as a debug tool.
*/
statcheck()
	{
	int i;
	struct port *p;

	for (i=0; i<20; i++) {
		printf("\n%d > ",i);
		p = portlist[i];
		if (p != NULL) 
			printf("state: %d  %5u  %5u  %10ld  %5d  %5d",
					p->state,intswap(p->tcpout.t.source),
					intswap(p->tcpout.t.dest),p->out.lasttime,p->rto,
					p->out.contain);
	}
}

/*************************************************************************/
/*  config network parameters
*   Set IRQ and DMA parameters for initialization of the 3com adaptor
*/
static int nnirq=3,nnaddr=0xd000,nnioaddr=0x300;

netparms(irq,address,ioaddr)
	int irq,address,ioaddr;
	{

	nnirq = irq;
	nnaddr = address;
	nnioaddr = ioaddr;

	return(0);
}

/**********************************************************************/
/* netconfig
*  load the function pointers for network access
*  Currently setaddr() is not used, so it isn't loaded.
*
*  Note that netparms is called BEFORE netconfig.  So if you have any
*  really special variables to set for your board that involve
*  irq,address and ioaddr, you can add calls to your special routines
*  in this section.
*
*  Some drivers will do the interrupt driver and board initialization
*  in etopen() and some will do it in getaddr().
*/
netconfig(s)
	char *s;
	{

	if (!strncmp(s,"ni5",3) || !strncmp(s,"mi",2)) {
		etopen = M5etopen;
		xmit = M5xmit;
		recv = M5recv;
		getaddr = M5getaddr;
		etupdate = M5etupdate;
		etclose = M5etclose;
/*
*   special initialization call would go here
*/
	}
	else if (!strncmp(s,"nicps",5)) {
		etopen = U2etopen;
		xmit = U2xmit;
		recv = U2recv;
		getaddr = U2getaddr;
		etupdate = U2etupdate;
		etclose = U2etclose;
	}
	else if (!strncmp(s,"nicpc",5) || !strncmp(s,"pcnic",5)) {
		etopen = U1etopen;
		xmit = U1xmit;
		recv = U1recv;
		getaddr = U1getaddr;
		etupdate = U1etupdate;
		etclose = U1etclose;
	}
	else if (!strncmp(s,"wd",2) || !strncmp(s,"800",3)) {
		etopen = WDetopen;
		xmit = WDxmit;
		recv = WDrecv;
		getaddr = WDgetaddr;
		etupdate = WDetupdate;
		etclose = WDetclose;
	}
	else if (!strncmp(s,"3c523",5) || !strncmp(s,"523",3)) {
		etopen = E2etopen;
		xmit = E2xmit;
		recv = E2recv;
		getaddr = E2getaddr;
		etupdate = E2etupdate;
		etclose = E2etclose;
	}
	else if (!strncmp(s,"r501",4)) {		/* special reserve driver */
		etopen = E3etopen;
		xmit = E3xmit;
		recv = E3recv;
		getaddr = E3getaddr;
		etupdate = E3etupdate;
		etclose = E3etclose;
	}
	else if (!strncmp(s,"3c",2) || 1) {		/* default choice */
		etopen = E1etopen;
		xmit = E1xmit;
		recv = E1recv;
		getaddr = E1getaddr;
		etupdate = E1etupdate;
		etclose = E1etclose;
	}

}

/**********************************************************************/
/*  netarpme
*   send an arp to my address.  arpinterpret will notice any response.
*   Checks for adapters which receive their own broadcast packets.
*/
netarpme(s)
	char *s;
	{

	if (etopen == U2etopen)
		return(0);
	if (etopen == U1etopen)
		return(0);

	reqarp(s);		/* send it */

	return(0);
}


/**********************************************************************/

initbuffer()
	{

	bufpt = bufread = buforg = raw;	/*  start at the beginning */

	bufend = &raw[14500];        		/* leave 2K breathing room, required */
	buflim = 12000;                      /* another 2K breathing room */

	(*getaddr)(nnmyaddr,nnaddr,nnioaddr);

	return(0);
}

/**********************************************************************/
/*   demux
*      find the packets in the buffer, determine their lowest level
*  packet type and call the correct interpretation routines
*
*  the 'all' parameter tells demux whether it should attempt to empty
*  the input packet buffer or return after the first packet is dealt with.
*
*  returns the number of packets demuxed
*/
demux(all)
	int all;
	{
	uint16 getcode;
	int nmuxed;
	DLAYER *firstlook;

	nmuxed = 0;
	if (!etupdate)					/* check that network is hooked up */
		return(0);

	do {							/* while all flag is on */

		(*recv)();					/* NULL operation for 3COM */

		if (bufbig > 0) {

			nmuxed++;
			firstlook = (DLAYER *)(bufread+2); 	/* where packet is */

			getcode = firstlook->type;			/* where does it belong? */

			switch (getcode) {					/* what to do with it? */
				case EARP:
				case ERARP:
					arpinterpret(firstlook);	/* handle ARP packet */
					break;
				case EIP:
					ipinterpret(firstlook);
					break;
				default:
					break;
			}

			(*etupdate)();   		/* update read pointers in buffer, free packet */

		}
		else 
			all = 0;


	} while (all);			/* should we look for more to deal with? */


	return(nmuxed);          /* no packets anymore */

}

/************************************************************************/
/*  dlayersend
*
*  usage:   err = dlayersend(ptr,size)
*      err = 0 for successful, non-zero error code otherwise
*      ptr is to a dlayer packet header
*      size is the number of bytes total
*
*  This particular dlayer routine is for Ethernet.  It will have to be
*  replaced for any other dlayer.
*
*  Ethernet addresses are resolved at higher levels because they will only
*  need to be resolved once per logical connection, instead of once per
*  packet.  Not too layer-like, but hopefully modular.
*
*/

dlayersend(ptr,size)
	DLAYER *ptr;
	unsigned size;
	{
	int ret;
	
	ret = (*xmit)(ptr,size);	/* send it out, pass back return code */
								/* xmit checks for size < 60 */

/*
*   automatic, immediate retry once
*/
	if (ret) {
		if (ret == (*xmit)(ptr,size))
			nnerror(100);		/* post user error message */
	}

	return(ret);
}

/***************************************************************************/
/* dlayerinit
*  Do machine dependent initializations of whatever hardware we have
*  (happens to be ethernet board here ) 
*/
dlayerinit()
	{

	if (initbuffer() || !etopen)
		return(-1);

	return((*etopen)(nnmyaddr,nnirq,nnaddr,nnioaddr));
}

dlayershut()
	{
	if (etclose)
		(*etclose)();
}

/***************************************************************************/
/*  pcgetaddr
*   return results from indirect getaddr call.
*   This is a pc-specific request for the 48-bit address which was added
*   so that the user program could print the value.
*/
pcgetaddr(s,x,y)
	char *s;
	int x,y;
	{
	if (getaddr)
		(*getaddr)(s,x,y);
}
